package com.alpha.lagouapk.ui.interview.ai.widget

import android.content.Context
import android.graphics.Canvas
import android.graphics.Color
import android.graphics.Paint
import android.graphics.RectF
import android.os.Vibrator
import android.text.TextUtils
import android.util.AttributeSet
import android.view.View
import com.alpha.lagouapk.R
import com.lagou.commonlibs.utils.ResourceUtils
import kotlin.random.Random

/**
 * Time：2024/8/2 14:31:04
 * Author：wangjunwei
 * Description：纯语音面试页面，录音交互自定义View
 *
 */
class PureVoiceWaveView @JvmOverloads constructor(
    context: Context,
    attrs: AttributeSet? = null,
    style: Int = 0
) : View(context, attrs, style) {

    private val mWaveItemWidth = ResourceUtils.getDimenSize(R.dimen.dp_7)//语音条的宽度
    private val mWaveItemMinHeight = ResourceUtils.getDimenSize(R.dimen.dp_7)//语音条的最小高度
    private val mWaveItemMaxHeight = ResourceUtils.getDimenSize(R.dimen.dp_30)//语音条最大高度
    private val mWaveItemGap = ResourceUtils.getDimenSize(R.dimen.dp_9)//语音条的间隔
    private val mWaveItemCorner = ResourceUtils.getDimenSize(R.dimen.dp_4)//语音条的圆角
    private val mWaveItemCount = 3 //语音条的数量
    private var mWavePaint: Paint = Paint(Paint.ANTI_ALIAS_FLAG)//绘制语音波纹画笔
    private val mBottomTipPaint = Paint(Paint.ANTI_ALIAS_FLAG)// 下边文案的画笔

    /**
     * 文本和语音波纹指示器间距
     */
    private val dp_10 = ResourceUtils.getDimenSize(R.dimen.dp_10)

    /**
     * 文案的高度
     */
    private val dp_22_05 = ResourceUtils.getDimenSize(R.dimen.dp_22_05)

    /**
     * 录音交互View最大高最小高
     */
    private val dp_90 = ResourceUtils.getDimenSize(R.dimen.dp_90)

    private var mState: State = State.UN_SPECK

    private var vibrator: Vibrator? = null

    /**
     * 倒计时文本
     */
    var countDownText: String? = null
        set(value) {
            if (mState == State.COUNT_DOWN_SPECK && TextUtils.equals(value, field)){
                return
            }

            mState = State.COUNT_DOWN_SPECK
            field = value
            invalidate()
        }

    /**
     * 语音波纹浮动系数，从0-1
     */
    var amplitude: Float = 0f
        set(value) {
            if (value == field){
                return
            }

            if (value > 1){
                field = 1f
            }else {
                field = value
            }
            invalidate()
        }

    enum class State(val value: Int) {
        WAIT_LISTEN(0), // 等待开启录音中
        UN_SPECK(1), // 录音中 - 未说话状态
        ON_SPECK(2), // 录音中 - 正在说话状态
        RECORD_PAUSE(3), // 录音暂停中
        COUNT_DOWN_SPECK(4), // 进入倒计时状态
        SENDING(5), // 发送语音中（停止输入语音2s 或者 达到最大语音输入时间）
        AI_SPECK(6), // Ai说的过程中，录音交互页面不展示（高度为0）
        AI_SPECK_PAUSE(7), // Ai说话暂停中
        SESSION_FINISH(8); // 会话已结束
    }

    init {
        mWavePaint.color = Color.parseColor("#FFFFFF")
        mWavePaint.strokeWidth = mWaveItemWidth.toFloat()
        mWavePaint.style = Paint.Style.FILL

        mBottomTipPaint.color = Color.parseColor("#cccccc")
        mBottomTipPaint.textSize = ResourceUtils.getDimenSize(R.dimen.sp_16).toFloat()

        vibrator = context.getSystemService(Context.VIBRATOR_SERVICE) as Vibrator;
    }

    private fun getViewWidth(): Int {
        return mWaveItemWidth * mWaveItemCount + (mWaveItemCount - 1) * mWaveItemGap
    }

    override fun onMeasure(widthMeasureSpec: Int, heightMeasureSpec: Int) {
        super.onMeasure(widthMeasureSpec, heightMeasureSpec)
        val heightSpecSize = MeasureSpec.getSize(heightMeasureSpec)
        val heightSpecMode = MeasureSpec.getMode(heightMeasureSpec)

        val widthSpecSize = MeasureSpec.getSize(widthMeasureSpec)
        val realHeight = if (mState == State.AI_SPECK) {
            dp_90
        } else {
            dp_90
        }
        if (heightSpecMode == MeasureSpec.AT_MOST || heightSpecMode == MeasureSpec.UNSPECIFIED) {
            setMeasuredDimension(widthSpecSize, realHeight)
        } else {
            setMeasuredDimension(widthSpecSize, heightSpecSize.coerceAtLeast(realHeight))
        }
    }

    override fun onDraw(canvas: Canvas) {
        super.onDraw(canvas)
        drawWave(canvas)
        drawBottomTip(canvas)
    }

    /**
     * 绘制语音波纹
     */
    private fun drawWave(canvas: Canvas) {
        if (mState == State.WAIT_LISTEN
            || mState == State.SENDING
            || mState == State.AI_SPECK
            || mState == State.AI_SPECK_PAUSE
            || mState == State.SESSION_FINISH) {
            return
        }
        val centerY = height - dp_22_05 - dp_10 - mWaveItemMaxHeight / 2
        val offsetX = width / 2f - getViewWidth() / 2f
        for (i in 0.until(mWaveItemCount)) {
            //wave 高度变幻最大值  最大高减去最小高
            val indicatorHeight = mWaveItemMinHeight + Random.nextInt(
                0,
                mWaveItemMaxHeight - mWaveItemMinHeight
            ) * amplitude
            val startX = i * (mWaveItemWidth + mWaveItemGap).toFloat() + offsetX
            canvas.drawRoundRect(
                RectF(
                    startX,
                    centerY - indicatorHeight / 2f,
                    startX + mWaveItemWidth,
                    centerY + indicatorHeight / 2f
                ), mWaveItemCorner.toFloat(), mWaveItemCorner.toFloat(), mWavePaint
            )
        }
    }

    /**
     * 绘制顶部提示
     */
    private fun drawBottomTip(canvas: Canvas) {
        if (mState == State.WAIT_LISTEN
            || mState == State.SENDING
            || mState == State.AI_SPECK
            || mState == State.AI_SPECK_PAUSE) {
            return
        }
        val text = when (mState) {
            State.SESSION_FINISH -> "没有进行中的面试"
            State.UN_SPECK -> "你可以开始说话"
            State.ON_SPECK -> "正在听"
            State.RECORD_PAUSE -> "录音暂停中"
            State.COUNT_DOWN_SPECK -> countDownText
            else -> ""
        } ?: ""

        if (TextUtils.isEmpty(text)) {
            return
        }

        val centerWaveY = height - dp_22_05 - dp_10 - mWaveItemMaxHeight / 2
        val textWidth = mBottomTipPaint.measureText(text)
        canvas.drawText(
            text,
            (canvas.width - textWidth) / 2,
            centerWaveY + mWaveItemMaxHeight / 2 + dp_10 + -mBottomTipPaint.fontMetrics.ascent,
            mBottomTipPaint
        )
    }

    /**
     * 触发振动
     */
    private fun triggerVibrate() {
        vibrator?.vibrate(100)
    }

    /***
     * 改变状态
     */
    fun changeState(state: State) {
        /**
         * 状态变更只能从
         */
        if (mState == state) {
            return
        }
        val isTriggerVibrate = (state == State.SENDING && mState == State.COUNT_DOWN_SPECK)
//        val isSizeChange = (mState == State.UN_SPECK && state != State.ON_SPECK && state != State.COUNT_DOWN_SPECK)
//                    || (mState == State.ON_SPECK && state != State.UN_SPECK && state != State.COUNT_DOWN_SPECK)
//                    || (mState == State.COUNT_DOWN_SPECK && state != State.UN_SPECK && state != State.ON_SPECK)

//        this.mState = state
//        if (isSizeChange) {
//            requestLayout()
//        } else {
//            invalidate()
//        }
        this.mState = state
        invalidate()
        if (isTriggerVibrate) {
            triggerVibrate()
        }
    }

    /**
     * 获取当前的状态
     */
    fun getCurrentState(): State{
        return mState
    }
}